require 'braintree'

module SimplePayment
  module Braintree
    class Gateway
      FEE_PERCENTAGE = 0.029
      FEE_FLAT = 0.30

      def initialize(settings={})
        @settings = settings
      end

      def new_token
        gateway.client_token.generate
      end

      def execute(payment, nonce)
        result = gateway.transaction.sale(execute_args(payment, nonce))
        [Request.new(:sale), Response.new(result)]
      rescue StandardError => e
        raise TransmissionError.new Request.new(:sale), e.message
      end

      def completed?(payment)
        transaction = gateway.transaction.find(payment.external_id)
        %w(settling settled).include?(transaction.status)
      rescue StandardError => e
        raise TransmissionError.new Request.new(:find), e.message
      end

      def cancel(payment)
        result = gateway.transaction.void(payment.external_id)
        [Request.new(:cancel), Response.new(result)]
      rescue StandardError => e
        raise TransmissionError.new Request.new(:cancel), e.message
      end

      def refund(payment)
        result = gateway.transaction.refund(payment.external_id)
        [Request.new(:refund), Response.new(result)]
      rescue StandardError => e
        raise TransmissionError.new Request.new(:refund), e.message
      end

      def calculate_fee(payment)
        raise 'payment cannot be nil' unless payment
        (payment.amount * FEE_PERCENTAGE) + FEE_FLAT
      end

      private

      def gateway
        ::Braintree::Gateway.new @settings
      end

      def execute_args(payment, nonce)
        {
          amount: payment.amount,
          payment_method_nonce: nonce,
          options: {
            submit_for_settlement: true
          }
        }.tap do |a|
          if payment.payable&.respond_to?(:custom_fields) &&
              payment.payable&.custom_fields.present?
            a[:custom_fields] = payment.payable.custom_fields
          end
        end
      end
    end
  end
end
